use std::process::Stdio;

use serde::{Deserialize, Serialize};
use tokio::process::Command;
use tokio_stream::StreamExt;
use tokio_util::codec::{BytesCodec, FramedRead};

#[derive(Deserialize)]
struct ExecReq {
    user: String,
    program: String,
    #[serde(default)]
    args: Vec<String>,
}

#[derive(Serialize)]
enum ExecOut {
    Stdout(String),
    Stderr(String),
    Exit(String),
}

impl TryFrom<async_nats::Message> for ExecReq {
    type Error = serde_json::Error;

    fn try_from(value: async_nats::Message) -> Result<Self, Self::Error> {
        serde_json::from_slice(&value.payload[..])
    }
}

impl ExecReq {
    async fn handle(&self, client: crate::messaging::Client) -> anyhow::Result<()> {
        let mut cmd = if self.user != "root" {
            let mut cmd = Command::new("sudo");
            cmd.arg("-iu").arg(&self.user).arg("--").arg(&self.program);
            cmd
        } else {
            Command::new(&self.program)
        };

        let mut io = cmd
            .args(&self.args)
            .stdout(Stdio::piped())
            .stderr(Stdio::piped())
            .spawn()?;

        let stdout =
            FramedRead::new(io.stdout.take().unwrap(), BytesCodec::new()).filter_map(|stdout| {
                stdout
                    .map(|bytes| ExecOut::Stdout(String::from_utf8_lossy(&bytes[..]).to_string()))
                    .map_err(|err| tracing::error!(%err, "read error on stdout"))
                    .ok()
            });

        let stderr =
            FramedRead::new(io.stderr.take().unwrap(), BytesCodec::new()).filter_map(|stderr| {
                stderr
                    .map(|bytes| ExecOut::Stderr(String::from_utf8_lossy(&bytes[..]).to_string()))
                    .map_err(|err| tracing::error!(%err, "read error on stderr"))
                    .ok()
            });

        let stream = stdout
            .merge(stderr)
            .chain(futures::FutureExt::into_stream(async move {
                io.wait()
                    .await
                    .map(|exit_status| ExecOut::Exit(exit_status.to_string()))
                    .unwrap()
            }));

        tokio::spawn(async move {
            tokio::pin!(stream);

            while let Some(resp) = stream.next().await {
                client
                    .publish("exec.reply", serde_json::to_vec(&resp).unwrap())
                    .await
                    .unwrap();
            }
        });

        Ok(())
    }
}

pub async fn init_exec_service(client: crate::messaging::Client) -> anyhow::Result<()> {
    let mut stream = client.subscribe::<ExecReq>("exec").await?;

    tokio::spawn(async move {
        while let Some(msg) = stream.next().await {
            msg.handle(client.clone()).await.unwrap();
        }
    });

    Ok(())
}
